package ProsperXsdParser;
use 5.006;
use Carp;

=head1 NAME

ProsperXsdParser - Prosper data export XSD file parser for Perl

=head1 SYNOPSIS

  use ProsperXsdParser;

  my $parser = new ProsperXsdParser($xsd_file);
  my $tables = $parser->{TABLES};
  for my $table_name (keys %{$tables}) {
      for (my $i = 0; $i < @$table; $i++) {
          my $column = $table->[$i];
          my $column_name = $column->[0];
          my $column_type = $column->[1];
          # process as appropriate
      }
  }

=head1 DESCRIPTION

This module parses the Prosper data export XSD file to extract table and
column definitions for a SQL database.  This version is intended for MySQL
but should be usable with other databases with fairly minor changes.  (See
comments in the code.)

=head1 NOTES

MySQL does not allow columns to be named "Key", so this
is always renamed to "ProsperKey".

=cut

use strict;

use XML::Parser;

my $indent_level = 0;
my %tables;
my $current_table;

sub new {
	my $class = shift;
	my $input_file = shift;
	if (! defined $input_file) {
		croak "usage: need to specify an input XML file to parse";
	}
	my $parser = new XML::Parser(Handlers => {Start => \&handle_start,
                                                End   => \&handle_end});
	$parser->parsefile($input_file);
	my $self  = {};
	$self->{TABLES}   = \%tables;
	bless ($self, $class);
	return $self;
}


sub get_type {
	my $attrs = shift;

	if (defined $attrs->{'sql:datatype'}) {
		# currently, this is always dateTime, which MySQL supports
		return $attrs->{'sql:datatype'};
	} elsif (defined $attrs->{'type'}) {
		# these are all for MySQL, change as appropriate
		my $type = $attrs->{'type'};
		$type =~ s/xsd://;
		if ($type eq 'integer') {
			return 'int';
		} elsif ($type eq 'string') {
			return 'varchar(300)';
		} elsif ($type eq 'decimal') {
			return 'decimal(13,5)';
		} elsif ($type eq 'boolean') {
			return 'boolean';
		}
	}
	croak "no type! $_";
}

sub handle_start {
        my ($xpat, $element, %attrs) = @_;

	if ($element eq 'xsd:complexType') {
		$indent_level++;
	} elsif ($element eq 'xsd:element') {
		if ($indent_level == 1) {
			$current_table = $attrs{'name'};
			$tables{$current_table} = [];
		} elsif ($indent_level == 3) {
			my $column_name = $attrs{'name'};
			if ($column_name eq 'Key') {
				$column_name = 'ProsperKey';
			}
			my $type = get_type(\%attrs);
			my $column = [];
			$column->[0] = $column_name;
			$column->[1] = $type;
			push(@{$tables{$current_table}}, $column);
		} elsif ($indent_level > 3) {
			croak "unknown indent level: $_ at $indent_level";
		}
	}
}

sub handle_end {
        my ($xpat, $element) = @_;

	if ($element eq 'xsd:complexType') {
		$indent_level--;
	}
}

# return true to make importer happy
1;
