#! /usr/bin/perl -w

=head1 NAME

xml2inserts.pl - import data from Prosper data export XML file into SQL database

=head1 SYNOPSIS

  xml2inserts.pl <input XML file>

Where "input XML file" is the name of a file containing the Prosper data
export.  There are typically two of these, one named ProsperDataExport.xml
and one ProsperDataPrivateExport.xml.

=head1 DESCRIPTION

This program reads the data from a Prosper data export file and writes SQL
to insert it into appropriate tables in a SQL database.   Output is
written to standard output -- this can be saved to a file or piped directly
into mysql if you are feeling lucky.

This is intended for MySQL but should be usable with other databases with
fairly minor changes.  (See comments in the code and in ProsperXsdParser.pm.)

=head1 SEE ALSO

xsd2mysql.pl -- used to create the tables used by this script

=cut

use strict;

use XML::Parser;
use ProsperXsdParser;

my $indent_level = 0;

if (@ARGV != 1 or ! -f $ARGV[0]) {
	die "usage: $0 <input XML file>\n";
}
my $input_file = $ARGV[0];
my $xsd_file = $ARGV[0];
$xsd_file =~ s/xml$/xsd/;

my $xsd = new ProsperXsdParser($xsd_file);

# We're using three callback functions to process the XML data, each named
# handle_<something>.  See XML::Parser POD for details for how these work.
my $parser = new XML::Parser(Handlers => {Start => \&handle_start,
						End   => \&handle_end,
						Char  => \&handle_char});

$parser->parsefile($ARGV[0]);

my $current_table;
my @current_columns;
my $current_column;
my $column_num;
my %current_values;
my %current_types;

sub handle_start {
	my ($xpat, $element, %attrs) = @_;
	$indent_level++;
	if ($indent_level == 2) {
		$current_table = $element;
	} elsif ($indent_level == 3) {
		$column_num = -1;
	} elsif ($indent_level == 4) {
		$current_column = $element;
		$column_num++;
		if ($current_column ne 'ModifiedDate') {
			if ($current_column eq 'Key') {
				$current_column = 'ProsperKey';
			}
			push @current_columns, $current_column;
		}
	}
}

sub handle_char {
	my ($xpat, $str) = @_;

	if ($indent_level == 4) {
		my $type = get_type();
		$current_types{$current_column} = $type;
		if ($type =~ /^varchar/) {
			if ($str =~ /\n/) {
				$str =~ s/\n/ /g;
			}
			if ($str =~ /'/) {
				$str =~ s/'/''/g;
			}
			if ($str =~ /\\/) {
				# this may be specific to MySQL
				$str =~ s/\\/\\\\/g;
			}
		} elsif ($type eq 'dateTime') {
			$str =~ s/T/ /;
		}
		if (! defined $current_values{$current_column}) {
			$current_values{$current_column} = '';
		}
		$current_values{$current_column} .= $str;
	}
}

sub handle_end {
	my ($xpat, $element) = @_;
	$indent_level--;

	if ($indent_level == 2) {
		print "INSERT INTO $current_table (\n ";
		for my $col (@current_columns) {
			print $col;
			if ($col ne @current_columns[@current_columns - 1]) {
				print ", ";
			}
		}
		print ")\n";
		print "VALUES (\n ";
		for my $col (@current_columns) {
			if (! defined $current_values{$col}) {
				print 'NULL';
			} else {
				my $type = $current_types{$col};
				if ($type =~ /^varchar/ or $type eq 'dateTime') {
					# With MySQL, date can be just a string
					# in the right format.
					print "'$current_values{$col}'";
				} else {
					print $current_values{$col};
				}
			}
			if ($col ne @current_columns[@current_columns - 1]) {
				print ", ";
			}
		}
		print ");\n";
		@current_columns = ();
		%current_values = ();
	}
}

sub get_type {
	$xsd->{TABLES}->{$current_table}->[$column_num][1];
}
